function [u] = evalAllLowLevConstrs(obj,i,x,y)
    %u=evalAllLowLevConstrs(i,x,y) plugs the three different types of lower
    %level constraints together. The first input is the index of the
    %indexset, x are the upper level variables and y the lower level
    %variables.    
    %The first constraints are the non-linear constratins, the next are the
    %linear constraints and the last are the boundaries. First come the 
    %lower bounds and then the upper bounds. We delete all boundaries with
    %value + or - infinity.
    
    u1=obj.evalLowLevNonlinConstrs(i,x,y);
    [A,b]=obj.getLinIneqsOnLowLevVars(i,x);
    if size(A,2)>0
        u2=A*y'-b;
        u2=u2';
    else
        u2=[];
    end
    
    %Add the boundaries
    [lb,ub]=obj.getBoundsOnLowLevVars(i,x);
    constrlb=lb-y;
    %drop the boundaries with -inf
    constrlb=constrlb(constrlb~=-inf);   
    construb=y-ub;
    construb=construb(construb~=-inf);
    
    u=[u1,u2,constrlb,construb];

end

