function [h] = plotOptValFunction( problem,i,IndicesFreePars,Bounds,ValsFixedPars,numberOfEvaluatedPoints)
    %h=<problemname>.plotOptValFunction(i,IndFree,Bounds,ValFixed,numberPoints)
    %plots the optimal value function. Where i is the index of the
    %semi-infinite constraint, IndFree contains the indices of the xi that 
    %can vary, the bounds for them are given in Bounds. all other values 
    %are given in ValFixed. The last input numberPoints is the number of
    %points that are evaluated. If i is a vector the maximum over all 
    %optimal value functions given in i is taken.

    % plots a part of the implicitly given function phi_i which is the
    % optimal value function of the lower level problem
    if size(i,1)~=1
        i=i';
    end
    assert (size(i,1)==1)
    for j=i
        assert(~mod(j,1),'please insert a index for ''i''')
        assert(1<=j && j<=problem.getNrSemiInfConstrs,'i is not the index of a semi-infinite constraint')
    end
    assert(size(IndicesFreePars,1)==1,'please insert a vector of size 1x1 or 1x2 for IndexDraw')
    n=size(IndicesFreePars,2);
    assert(size(Bounds,1)==2,'please insert the lower bounds in the first row and upper bounds in the second row')
    assert(n==size(Bounds,2),'the number of bounds have to coincide with the number of variables that can vary')
    switch n
        case 1
            h=implicitPlotOfOptValueFunction1Variable(problem,i,IndicesFreePars,Bounds,ValsFixedPars,numberOfEvaluatedPoints);
        case 2
            h=implicitPlotOfOptValueFunction2Variables(problem,i,IndicesFreePars,Bounds,ValsFixedPars,numberOfEvaluatedPoints);
        otherwise
            error('we can only plot the function for 1 or 2 variables')
    end
end
