function [ fh ] = plotFeasibleSetOfGSIP2D(problem,indicesFreeParameters,valuesFixedParameters,lowerBounds,upperBounds,nrPointsInDir)

%Global settings
NrSemiInfConstrs=problem.getNrSemiInfConstrs();
NrPointsInDir=nrPointsInDir;
if NrPointsInDir>100
    warning('''NrPointsInDir^2'' lower level problems have to be solved. This may take some time')
end
y1Range = upperBounds(1)-lowerBounds(1);
y2Range = upperBounds(2)-lowerBounds(2);

width = 14;
height = y2Range/y1Range*width;

fh = fig('width',14, 'height', height);
hold on

%Settings for the solver
myLLPfminconOptions = optimset('Algorithm', 'sqp');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'Display', 'off');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxFunEvals', Inf);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxIter', 1000);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FunValCheck', 'on');
%myLLPfminconOptions = optimset(myLLPfminconOptions, 'GradObj', 'on', 'GradConstr', 'on', 'DerivativeCheck', der_check);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'ScaleProblem', 'obj-and-constr');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FinDiffType', 'central');

%compute x data
x1_temp=linspace(lowerBounds(1),upperBounds(1),NrPointsInDir);
x2_temp=linspace(lowerBounds(2),upperBounds(2),NrPointsInDir);
[x1,x2]=meshgrid(x1_temp,x2_temp);

%initialize phi
phi=-inf*ones(NrPointsInDir);

%check if indices ar in the right order
if indicesFreeParameters(1)>indicesFreeParameters(2)
    temp=indicesFreeParameters(1);
    indicesFreeParameters(1)=indicesFreeParameters(2);
    indicesFreeParameters(2)=temp;
    temp=lowerBounds(:,1);
    lowerBounds(:,1)=lowerBounds(:,2);
    lowerBounds(:,2)=temp;
    temp=upperBounds(:,1);
    upperBounds(:,1)=upperBounds(:,2);
    upperBounds(:,2)=temp;  
end

%check wether there are equality constraints
[A,b]=problem.getLinEqsOnUppLevVars();
if size(A,1)>0||problem.getNrUppLevNonlinEqs>0
    warning('This plot does not work for equality constraints. We ignore the equality constraints');            
end
        
        
for j=1:NrPointsInDir
    for j2=1:NrPointsInDir
        %Compute x-Point
        x=[valuesFixedParameters(1:indicesFreeParameters(1)-1),x1(j,j2),...
            valuesFixedParameters(indicesFreeParameters(1):indicesFreeParameters(2)-2),x2(j,j2),...
            valuesFixedParameters(indicesFreeParameters(2)-1:end)];
        %optimization of every lower level problem
        for i=1:NrSemiInfConstrs
            k=problem.getIndexSetOfSemiInfConstr(i);   
            Yint = problem.getInitValsLowLevVars(k,x);
            [lbl, ubl] = problem.getBoundsOnLowLevVars(k,x);

            [Aineq,bineq] = problem.getLinIneqsOnLowLevVars (k,x);
            % max-problem    
            [~,fval] = fmincon(@LLPObjFun,Yint,Aineq,bineq,[],[],lbl,ubl,@LLPConFuns,myLLPfminconOptions,problem,i,x,k);
            phi(j,j2)=max(phi(j,j2),-fval);               
        end
        % non semi infinite constraints

        %compute non semi-infinite constraints
        %collect bounds
        [problb,probub]=problem.getBoundsOnUppLevVars();
        phi(j,j2)=max(phi(j,j2),problb(1)-x1(j,j2)); 
        phi(j,j2)=max(phi(j,j2),x1(j,j2)-probub(1));
        phi(j,j2)=max(phi(j,j2),problb(2)-x2(j,j2)); 
        phi(j,j2)=max(phi(j,j2),x2(j,j2)-probub(2));
        

        %collect linear constraints
        [A,b]=problem.getLinIneqsOnUppLevVars();
        for i2=1:size(A,1)
            phi(j,j2)=max(phi(j,j2),x*A(i2,:)'-b(i2));
        end


        if problem.getNrUppLevNonlinIneqs>0
            phi(j,j2)=max([phi(j,j2);problem.evalUppLevNonlinIneqConstrs(x)]);
        end

        %display every 10th evalutation of the function phi
        if mod((j-1)*NrPointsInDir+j2,10)==0
            fprintf('The first %i points are computed\n',(j-1)*NrPointsInDir+j2);
        end
    end
end

phiPosNeg=phi<=0;
o=ones(NrPointsInDir);
Z=zeros(NrPointsInDir);
Z(phiPosNeg)=1*o(phiPosNeg);
contourf(x1,x2,Z,[1,1]);

%Set the colormap to get the right fillings
cmap=[135/255, 206/255, 250/255];
set(fh,'colormap',cmap)

%settings for plot
grid on;

set(gca, 'DataAspectRatio', [1 1 1]);
set(gca, 'Units', 'centimeters');
xlim([lowerBounds(1), upperBounds(1)]);
xlabel(sprintf('x%i',indicesFreeParameters(1)));
ylabel(sprintf('x%i',indicesFreeParameters(2)));
ylim([lowerBounds(2), upperBounds(2)]);
title('Feasible set of GSIP')
hold off



%Functions for the optimization
    function [g, grad_g_y] = LLPObjFun(y,GSIP_problem2,i,xval,~)

        g_temp = GSIP_problem2.evalSemiInfConstrs(i,xval,y);
        g=-g_temp; %max-problem


        grad_g_y_temp = GSIP_problem2.DySemiInfConstrs(i,xval,y);
        grad_g_y=-grad_g_y_temp';
    end

    %constraints of the lower level problem
    function [u, h, grad_u, grad_h] = LLPConFuns(y,GSIP_problem2,~,xval,k2)

        h = [];
        grad_h = [];
        assert(size(y,1)==1);
        u = GSIP_problem2.evalLowLevNonlinConstrs(k2,xval,y);

        grad_u = GSIP_problem2.DyLowLevNonlinConstrs(k2,xval,y)';


    end


end

