classdef Container
    %Container is an abstract class. This class is the super class for all
    %containers. It contains some get functions and standard derivative
    %functions.
    
    properties (Abstract, GetAccess=protected)
        dim;                                 
        nrOfConstFuns;                       
        BoundingBox;                         
        
    end
    %containment functions
    methods (Abstract)
        %g=evalDescrFuns(i,y) evaluates the i-th describing function of the
        %container
        g=evalDescrFuns(obj,i,y)               
    end
    
    methods
        %Derivatives              
        function Dy_g=DyDescrFuns(obj,i,y0)
            %Dy_g=DyDescrFuns(i,y) returns the derivative (as Jacobian) of
            %the i-th descirbing function. If no funciton is implemented, a
            %numerical solution will be computed.
            Dy_g=getDerivativeViaDifferenceQuotient(@(y) obj.evalDescrFuns(i,y),y0,{1});
        end
        function DyDy_g=DyDyDescrFuns(obj,i,y0)
            %DyDy_g=DyDyDescrFuns(i,y)m returns the Hessian of the i-th
            %describing function. If no funciton is implemented, a
            %numerical solution will be computed. 
            DyDy_g=getSecDerivativeViaDifferenceQuotient(@(y) obj.evalDescrFuns(i,y),y0,{[]});
        end
    end
    
    methods
        %get functions
        function d= getDim(obj)
            %d=getDim() returns the dimension of the container.
            d=obj.dim;
        end
        function n=getNrDescrFuns(obj)
            %n=getNrDescrFuns() returns the number of functions that
            %describe the container
            n=obj.nrOfConstFuns;            
        end
        function [lb,ub]=getBoundingBox(obj)
            %[lb,ub]=getBoundingBox(obj) returns a bounding box of the
            %container. lb and ub ar row vectors which describe the lower
            % and upper bounds.
            lb=obj.BoundingBox(1,:);
            ub=obj.BoundingBox(2,:);
        end
     end
    
    
end

